'use strict';

var aesthetic = require('aesthetic');
var aestheticUtils = require('aesthetic-utils');
var fela = require('fela');
var felaDom = require('fela-dom');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var FelaAdapter = function (_Adapter) {
  _inheritsLoose(FelaAdapter, _Adapter);

  function FelaAdapter(fela) {
    var _this;

    _this = _Adapter.call(this) || this;

    _defineProperty(_assertThisInitialized(_this), "fela", void 0);

    _defineProperty(_assertThisInitialized(_this), "keyframes", {});

    _defineProperty(_assertThisInitialized(_this), "handleCss", function (css) {
      _this.fela.renderStatic(css);
    });

    _defineProperty(_assertThisInitialized(_this), "handleFallback", function (ruleset, name, value) {
      ruleset.addProperty(name, [].concat(value, aestheticUtils.toArray(ruleset.properties[name])));
    });

    _defineProperty(_assertThisInitialized(_this), "handleFontFace", function (sheet, fontFaces, fontFamily, srcPaths) {
      fontFaces.forEach(function (face, i) {
        var _ref = face.toObject(),
            local = _ref.local,
            style = _objectWithoutPropertiesLoose(_ref, ["local"]);

        _this.fela.renderFont(fontFamily, srcPaths[i], _extends({}, style, {
          localAlias: local
        }));
      });
    });

    _defineProperty(_assertThisInitialized(_this), "handleGlobal", function (sheet, selector, ruleset) {
      _this.fela.renderStatic(ruleset.toObject(), selector);
    });

    _defineProperty(_assertThisInitialized(_this), "handleKeyframe", function (sheet, keyframe, animationName) {
      _this.keyframes[animationName] = _this.fela.renderKeyframe(function () {
        return keyframe.toObject();
      }, {});
    });

    _defineProperty(_assertThisInitialized(_this), "handleMedia", function (ruleset, query, value) {
      ruleset.addNested("@media " + query, value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleNested", function (ruleset, selector, value) {
      ruleset.addNested(selector, value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleProperty", function (ruleset, name, value) {
      if (name === 'animationName') {
        ruleset.addProperty(name, _this.syntax.injectKeyframes(String(value), _this.keyframes).join(', '));
      } else {
        ruleset.addProperty(name, value);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleSupport", function (ruleset, query, value) {
      ruleset.addNested("@supports " + query, value);
    });

    _this.fela = fela;

    _this.syntax.on('attribute', _this.handleNested).on('css', _this.handleCss).on('fallback', _this.handleFallback).on('font-face', _this.handleFontFace).on('global', _this.handleGlobal).on('keyframe', _this.handleKeyframe).on('media', _this.handleMedia).on('property', _this.handleProperty).on('pseudo', _this.handleNested).on('selector', _this.handleNested).on('support', _this.handleSupport);

    return _this;
  }

  var _proto = FelaAdapter.prototype;

  _proto.flushStyles = function flushStyles() {
    felaDom.render(this.fela);
  };

  _proto.transformToClassName = function transformToClassName(styles) {
    return this.fela.renderRule(fela.combineRules.apply(void 0, styles.map(function (style) {
      return function () {
        return style;
      };
    })), {});
  };

  _proto.purgeStyles = function purgeStyles() {
    aestheticUtils.purgeStyles(aestheticUtils.getStyleElements('data-fela-type'), true);
  };

  return FelaAdapter;
}(aesthetic.Adapter);

/**
 * @copyright   2017-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */

module.exports = FelaAdapter;
